@@ -497,6 +497,9 @@ ibus_wayland_im_update_preedit_style (IBusWaylandIM *wlim)
             default:; /* Custom */
             }
             break;
+        case IBUS_ATTR_TYPE_HINT:
+            istyle = attr->value;
+            break;
         default:
             istyle = IBUS_ATTR_PREEDIT_NONE;
         }
@@ -957,11 +960,13 @@ ibus_wayland_seat_destroy (gpointer data)
 {
     IBusWaylandSeat *seat = (IBusWaylandSeat *)data;
     g_return_if_fail (seat);
-    g_free (seat->name);
-    zwp_input_popup_surface_v2_destroy (seat->input_popup_surface);
-    zwp_virtual_keyboard_v1_destroy (seat->virtual_keyboard);
-    zwp_input_method_keyboard_grab_v2_destroy (seat->keyboard_v2);
-    zwp_input_method_v2_destroy (seat->input_method_v2);
+    g_clear_pointer (&seat->name, g_free);
+    g_clear_pointer (&seat->input_popup_surface,
+                     zwp_input_popup_surface_v2_destroy);
+    g_clear_pointer (&seat->virtual_keyboard, zwp_virtual_keyboard_v1_destroy);
+    g_clear_pointer (&seat->keyboard_v2,
+                     zwp_input_method_keyboard_grab_v2_destroy);
+    g_clear_pointer (&seat->input_method_v2, zwp_input_method_v2_destroy);
     g_slice_free (IBusWaylandSeat, seat);
 }
 
@@ -1332,17 +1337,6 @@ _process_key_event_repeat_delay_cb (gpointer user_data)
     /* The key release event was sent to non-Wayland apps likes xterm. */
     if (!priv->ibuscontext) {
         event->count_cb_id = 0;
-        /* Stop the infinite Return keys in non-Wayland apps likes xterm in
-         * the Sway desktop session.
-         * But priv->context in NULL in the Wayland input-method V1 here.
-         */
-        if (priv->version != INPUT_METHOD_V1) {
-            ibus_wayland_im_key(event->wlim,
-                                event->serial,
-                                event->time + priv->repeat_delay,
-                                event->key,
-                                WL_KEYBOARD_KEY_STATE_RELEASED);
-        }
         return G_SOURCE_REMOVE;
     }
     /* The focus is changed. */
@@ -1776,6 +1770,8 @@ _create_input_context_done (GObject      *object,
         ibus_input_context_set_capabilities (priv->ibuscontext,
                                              capabilities);
         ibus_input_context_set_client_commit_preedit (priv->ibuscontext, TRUE);
+        ibus_input_context_set_preedit_format (priv->ibuscontext,
+                                               IBUS_PREEDIT_FORMAT_HINT);
         if (_use_sync_mode == 1) {
             ibus_input_context_set_post_process_key_event (priv->ibuscontext,
                                                            TRUE);
@@ -1818,6 +1814,9 @@ input_method_activate (void                               *data,
                                   wlim);
         break;
     case INPUT_METHOD_V2:
+        priv->seat->virtual_keyboard =
+                zwp_virtual_keyboard_manager_v1_create_virtual_keyboard (
+                        _virtual_keyboard_manager, priv->seat->seat);
         priv->seat->keyboard_v2 = zwp_input_method_v2_grab_keyboard (
                 input_method->u.input_method_v2);
         zwp_input_method_keyboard_grab_v2_add_listener (
@@ -1862,8 +1861,8 @@ input_method_deactivate (void                               *data,
 
 
     if (priv->ibuscontext) {
-        const gchar *object_path = g_dbus_proxy_get_object_path (
-                G_DBUS_PROXY (priv->ibuscontext));
+        gchar *object_path = g_strdup (g_dbus_proxy_get_object_path (
+                G_DBUS_PROXY (priv->ibuscontext)));
         ibus_input_context_focus_out (priv->ibuscontext);
         g_signal_handlers_disconnect_by_func (
                 priv->ibuscontext,
@@ -1891,6 +1890,7 @@ input_method_deactivate (void                               *data,
                        wayland_im_signals[IBUS_FOCUS_OUT],
                        0,
                        object_path);
+        g_free (object_path);
     }
 
     if (priv->preedit_text)
@@ -1904,8 +1904,14 @@ input_method_deactivate (void                               *data,
         }
         break;
     case INPUT_METHOD_V2:
-        if (priv->seat->keyboard_v2)
-            zwp_input_method_keyboard_grab_v2_release (priv->seat->keyboard_v2);
+        if (priv->seat->keyboard_v2) {
+            g_clear_pointer (&priv->seat->keyboard_v2,
+                             zwp_input_method_keyboard_grab_v2_release);
+        }
+        if (priv->seat->virtual_keyboard) {
+            g_clear_pointer (&priv->seat->virtual_keyboard,
+                             zwp_virtual_keyboard_v1_destroy);
+        }
         break;
     default:
         g_assert_not_reached ();
@@ -2120,10 +2126,6 @@ registry_handle_global (void               *data,
                             seat->seat);
             zwp_input_method_v2_add_listener (seat->input_method_v2,
                                               &input_method_listener_v2, wlim);
-            seat->virtual_keyboard =
-                    zwp_virtual_keyboard_manager_v1_create_virtual_keyboard (
-                            _virtual_keyboard_manager,
-                            seat->seat);
         }
         priv->seat = seat;
     } else if (!g_strcmp0 (interface,
@@ -2360,10 +2362,6 @@ ibus_wayland_im_constructor (GType                  type,
                 seat->seat);
         zwp_input_method_v2_add_listener (seat->input_method_v2,
                                           &input_method_listener_v2, wlim);
-        seat->virtual_keyboard =
-                zwp_virtual_keyboard_manager_v1_create_virtual_keyboard (
-                        _virtual_keyboard_manager,
-                        seat->seat);
     }
     if ((!seat || !seat->input_method_v2) && !priv->input_method_v1) {
         g_error ("No input_method global\n");
