@@ -3,7 +3,7 @@
 /* ibus - The Input Bus
  * Copyright (C) 2014 Peng Huang <shawn.p.huang@gmail.com>
  * Copyright (C) 2015-2025 Takao Fujiwara <takao.fujiwara1@gmail.com>
- * Copyright (C) 2014-2017 Red Hat, Inc.
+ * Copyright (C) 2014-2025 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -36,6 +36,7 @@
 
 #include <memory.h>
 #include <stdlib.h>
+#include <glib/gi18n-lib.h>
 
 #define IBUS_ENGINE_SIMPLE_GET_PRIVATE(o)  \
    ((IBusEngineSimplePrivate *)ibus_engine_simple_get_instance_private (o))
@@ -74,6 +75,7 @@ struct _IBusEngineSimplePrivate {
     GString            *tentative_match;
     int                 tentative_match_len;
     char               *tentative_emoji;
+    char               *client;
 
     guint               hex_mode_enabled : 1;
     guint               in_hex_sequence : 1;
@@ -84,6 +86,8 @@ struct _IBusEngineSimplePrivate {
     IBusLookupTable    *lookup_table;
     gboolean            lookup_table_visible;
     IBusText           *updated_preedit;
+    gboolean            do_inform_user_error;
+    guint               inform_user_error_timeout_id;
 };
 
 guint COMPOSE_BUFFER_SIZE = 20;
@@ -95,7 +99,13 @@ static IBusComposeTableEx *en_compose_table;
 /* functions prototype */
 static void     ibus_engine_simple_destroy      (IBusEngineSimple   *simple);
 static void     ibus_engine_simple_focus_in     (IBusEngine         *engine);
+static void     ibus_engine_simple_focus_in_id  (IBusEngine         *engine,
+                                                 const gchar
+                                                                   *object_path,
+                                                 const gchar        *client);
 static void     ibus_engine_simple_focus_out    (IBusEngine         *engine);
+static void     ibus_engine_simple_focus_out_id (IBusEngine         *engine,
+                                                 const gchar        *client);
 static void     ibus_engine_simple_reset        (IBusEngine         *engine);
 static gboolean ibus_engine_simple_process_key_event
                                                 (IBusEngine         *engine,
@@ -129,12 +139,17 @@ ibus_engine_simple_class_init (IBusEngineSimpleClass *class)
     GError *error = NULL;
     const char *contents;
     gsize length = 0;
+    guint16 saved_version = 0;
 
     ibus_object_class->destroy =
         (IBusObjectDestroyFunc) ibus_engine_simple_destroy;
 
     engine_class->focus_in  = ibus_engine_simple_focus_in;
+    engine_class->focus_in_id
+                            = ibus_engine_simple_focus_in_id;
     engine_class->focus_out = ibus_engine_simple_focus_out;
+    engine_class->focus_out_id
+                            = ibus_engine_simple_focus_out_id;
     engine_class->reset     = ibus_engine_simple_reset;
     engine_class->process_key_event
                             = ibus_engine_simple_process_key_event;
@@ -154,9 +169,33 @@ ibus_engine_simple_class_init (IBusEngineSimpleClass *class)
         return;
     }
     contents = g_bytes_get_data (data, &length);
-    en_compose_table = ibus_compose_table_deserialize (contents, length);
+    en_compose_table = ibus_compose_table_deserialize (contents,
+                                                       length,
+                                                       &saved_version);
+    g_bytes_unref (data);
+    if (!en_compose_table && saved_version) {
+        g_warning ("Failed to parse the builtin compose due to the different "
+                   "version %u. Please rebuild IBus resource files.",
+                   saved_version);
+    }
 }
 
+
+static gboolean
+inform_user_error_timeout_cb (gpointer user_data)
+{
+    IBusEngineSimple *simple = (IBusEngineSimple *)user_data;
+    IBusEngineSimplePrivate *priv;
+
+    g_return_val_if_fail (IBUS_IS_ENGINE_SIMPLE (simple), G_SOURCE_REMOVE);
+    priv = simple->priv;
+    priv->do_inform_user_error = FALSE;
+    priv->inform_user_error_timeout_id = 0;
+    ibus_engine_show_preedit_text ((IBusEngine *)simple);
+    return G_SOURCE_REMOVE;
+}
+
+
 static void
 ibus_engine_simple_init (IBusEngineSimple *simple)
 {
@@ -204,19 +243,99 @@ ibus_engine_simple_destroy (IBusEngineSimple *simple)
         IBUS_OBJECT (simple));
 }
 
+
+static void
+ibus_engine_simple_send_message_with_code (IBusEngineSimple *simple,
+                                           IBusEngineMsgCode code,
+                                           GError           *error)
+{
+    IBusEngineSimplePrivate *priv;
+    IBusMessage *message;
+
+    g_return_if_fail (IBUS_IS_ENGINE_SIMPLE (simple));
+
+    priv = simple->priv;
+    switch (code) {
+    case IBUS_ENGINE_MSG_CODE_INVALID_COMPOSE_SEQUENCE:
+        priv->do_inform_user_error = TRUE;
+        message = ibus_message_new (
+                IBUS_MESSAGE_DOMAIN_ENGINE,
+                code,
+                _("Detect unregistered character in your compose sequence"),
+                _("The character you just input is not recognized as a valid " \
+                  "part of the currently active compose sequence and the " \
+                  "character was cancelled. Try inputting the correct " \
+                  "character to the compose sequence again, or press Escape " \
+                  "key to terminate whole the compose sequence."),
+                  "timeout", 5,
+                  NULL);
+        break;
+    case IBUS_ENGINE_MSG_CODE_UPDATE_COMPOSE_TABLE:
+        message = ibus_message_new (
+                IBUS_MESSAGE_DOMAIN_ENGINE,
+                code,
+                _("Compose file update"),
+                error ? error->message : "",
+                "timeout", 120,
+                NULL);
+        break;
+    default:
+        g_assert_not_reached ();
+    }
+    ibus_engine_send_message (IBUS_ENGINE (simple), message);
+}
+
+
 static void
 ibus_engine_simple_focus_in (IBusEngine *engine)
 {
+    ibus_engine_simple_focus_in_id (engine, NULL, NULL);
+}
+
+
+static void
+ibus_engine_simple_focus_in_id (IBusEngine  *engine,
+                                const gchar *object_path,
+                                const gchar *client)
+{
+    /* Do not enable IBusEngine:has-focus-id in the IBusEngineSimple's
+     * constructor because the property should be enabled by the inherited
+     * engine. E.g. IBusEngineHangul disables it.
+     * The property is handled by ibusengine.c:_ibus_engine_has_focus_id();
+     */
+    if (IBUS_IS_ENGINE_SIMPLE (engine)) {
+        IBusEngineSimple *simple = IBUS_ENGINE_SIMPLE (engine);
+        g_free (simple->priv->client);
+        simple->priv->client = g_strdup (client);
+    } else {
+        g_warning ("IBUS_IS_ENGINE_SIMPLE(engine) in %s", G_STRFUNC);
+    }
     IBUS_ENGINE_CLASS (ibus_engine_simple_parent_class)->focus_in (engine);
 }
 
+
 static void
 ibus_engine_simple_focus_out (IBusEngine *engine)
 {
-    ibus_engine_simple_reset (engine);
+    ibus_engine_simple_focus_out_id (engine, NULL);
+}
+
+
+static void
+ibus_engine_simple_focus_out_id (IBusEngine  *engine,
+                                 const gchar *object_path)
+{
+    if (IBUS_IS_ENGINE_SIMPLE (engine)) {
+        IBusEngineSimple *simple = IBUS_ENGINE_SIMPLE (engine);
+        g_clear_pointer (&simple->priv->client, g_free);
+        ibus_engine_simple_reset (engine);
+    } else {
+        g_warning ("IBUS_IS_ENGINE_SIMPLE(engine) in %s", G_STRFUNC);
+    }
     IBUS_ENGINE_CLASS (ibus_engine_simple_parent_class)->focus_out (engine);
 }
 
+
 static void
 ibus_engine_simple_reset (IBusEngine *engine)
 {
@@ -237,11 +356,17 @@ ibus_engine_simple_reset (IBusEngine *engine)
         priv->tentative_match_len = 0;
     }
     ibus_engine_hide_preedit_text ((IBusEngine *)simple);
+    priv->do_inform_user_error = FALSE;
+    if (priv->inform_user_error_timeout_id) {
+        g_source_remove (priv->inform_user_error_timeout_id);
+        priv->inform_user_error_timeout_id = 0;
+    }
     g_object_unref (priv->updated_preedit);
     priv->updated_preedit =
             (IBusText *)g_object_ref_sink (updated_preedit_empty);
 }
 
+
 static void
 ibus_engine_simple_commit_char (IBusEngineSimple *simple,
                                 gunichar          ch)
@@ -304,6 +429,7 @@ ibus_engine_simple_commit_str (IBusEngineSimple *simple,
     g_free (backup_str);
 }
 
+
 static void
 ibus_engine_simple_update_preedit_text (IBusEngineSimple *simple)
 {
@@ -371,15 +497,59 @@ ibus_engine_simple_update_preedit_text (IBusEngineSimple *simple)
                         g_string_append_unichar (s, ch);
                     }
                 } else {
+                    guint unknown_ch = 0;
                     ch = ibus_keyval_to_unicode (keysym);
                     if (ch) {
-                        g_string_append_unichar(s, ch);
+                        /* Should provide ibus_unicode_get_script() for
+                         * other IBus engines?
+                         */
+                        if (g_unichar_get_script (ch) !=
+                            G_UNICODE_SCRIPT_UNKNOWN) {
+                            g_string_append_unichar(s, ch);
+                        } else {
+                            unknown_ch = ch;
+                        }
                     /* Can send Unicode char as keysym with <Uxxxx> format
                      * in comopse sequences and should not warn this case.
                      */
-                    } else if (g_unichar_validate (keysym)) {
-                        ch = keysym;
-                        g_string_append_unichar(s, ch);
+                    } else if (g_unichar_validate (keysym & 0xffff)) {
+                        ch = keysym & 0xffff;
+                        if (g_unichar_get_script (ch) !=
+                            G_UNICODE_SCRIPT_UNKNOWN) {
+                            g_string_append_unichar(s, ch);
+                        } else {
+                            unknown_ch = ch;
+                        }
+                    }
+                    if (unknown_ch) {
+                        gchar *name = NULL;
+                        gchar *layout = NULL;
+                        gchar *layout_end;
+                        g_object_get (simple, "engine-name", &name, NULL);
+                        if (!g_ascii_strncasecmp (name, "xkb:", 4)) {
+                            layout_end = name + 4;
+                            while (*layout_end && *layout_end != ':')
+                                layout_end++;
+                            if (*layout_end == ':') {
+                                layout = g_strndup (name + 4,
+                                                    layout_end - name - 4);
+                            }
+                        } else {
+                            g_warning ("Unexpected engine is used: %s", name);
+                        }
+                        g_free (name);
+                        if (layout) {
+                            ch = ibus_keysym_to_unicode_with_layout (keysym,
+                                                                     FALSE,
+                                                                     NULL,
+                                                                     layout,
+                                                                     NULL);
+                            g_free (layout);
+                        }
+                        if (ch)
+                            g_string_append_unichar(s, ch);
+                        else
+                            g_string_append_unichar(s, 0x00b7); /* · */
                     }
                 }
                 if (!ch) {
@@ -404,6 +574,11 @@ ibus_engine_simple_update_preedit_text (IBusEngineSimple *simple)
             g_object_unref (priv->updated_preedit);
             priv->updated_preedit =
                     (IBusText *)g_object_ref_sink (updated_preedit_empty);
+            priv->do_inform_user_error = FALSE;
+            if (priv->inform_user_error_timeout_id) {
+                g_source_remove (priv->inform_user_error_timeout_id);
+                priv->inform_user_error_timeout_id = 0;
+            }
         }
     } else if (s->len >= G_MAXINT) {
         g_warning ("%s is too long compose length: %lu", s->str, s->len);
@@ -411,9 +586,30 @@ ibus_engine_simple_update_preedit_text (IBusEngineSimple *simple)
         guint len = (guint)g_utf8_strlen (s->str, -1);
         IBusText *text = ibus_text_new_from_string (s->str);
         ibus_text_append_attribute (text,
-                IBUS_ATTR_TYPE_UNDERLINE, IBUS_ATTR_UNDERLINE_SINGLE, 0, len);
+                                    IBUS_ATTR_TYPE_UNDERLINE,
+                                    IBUS_ATTR_UNDERLINE_SINGLE,
+                                    0,
+                                    len);
         g_object_ref_sink (text);
-        ibus_engine_update_preedit_text ((IBusEngine *)simple, text, len, TRUE);
+        /* gnome-shell does not handle "SendMessageReceived" D-Bus method yet.
+         * Seems other Wayland desktops do not implement xdg-system-bell
+         * Wayland protocol yet.
+         */
+        if (priv->do_inform_user_error && priv->client &&
+            (!g_ascii_strncasecmp (priv->client, "gnome-shell", 11) ||
+             !g_ascii_strncasecmp (priv->client, "wayland", 7))
+           ) {
+            ibus_engine_hide_preedit_text ((IBusEngine *)simple);
+            if (priv->inform_user_error_timeout_id)
+                g_source_remove (priv->inform_user_error_timeout_id);
+            priv->inform_user_error_timeout_id =
+                    g_timeout_add (500,
+                                   inform_user_error_timeout_cb,
+                                   simple);
+        } else {
+            ibus_engine_update_preedit_text ((IBusEngine *)simple,
+                                             text, len, TRUE);
+        }
         g_object_unref (priv->updated_preedit);
         priv->updated_preedit = text;
     }
@@ -438,6 +634,7 @@ ibus_engine_simple_update_preedit_text (IBusEngineSimple *simple)
  */
 #define HEX_MOD_MASK (IBUS_CONTROL_MASK | IBUS_SHIFT_MASK)
 
+
 static gboolean
 check_hex (IBusEngineSimple *simple,
            int               n_compose)
@@ -501,6 +698,7 @@ check_hex (IBusEngineSimple *simple,
     return TRUE;
 }
 
+
 static IBusEngineDict *
 load_emoji_dict (void)
 {
@@ -524,6 +722,7 @@ load_emoji_dict (void)
     return emoji_dict;
 }
 
+
 static gboolean
 check_emoji_table (IBusEngineSimple       *simple,
                    int                     n_compose,
@@ -677,7 +876,10 @@ no_sequence_matches (IBusEngineSimple *simple,
         priv->compose_buffer[0] = 0;
         if (n_compose > 1) {
             /* Invalid sequence */
-            /* FIXME beep_window (event->window); */
+            ibus_engine_simple_send_message_with_code (
+                    simple,
+                    IBUS_ENGINE_MSG_CODE_INVALID_COMPOSE_SEQUENCE,
+                    NULL);
             ibus_engine_simple_update_preedit_text (simple);
             return TRUE;
         }
@@ -703,6 +905,7 @@ no_sequence_matches (IBusEngineSimple *simple,
     return FALSE;
 }
 
+
 static gboolean
 is_hex_keyval (guint keyval)
 {
@@ -711,6 +914,7 @@ is_hex_keyval (guint keyval)
   return g_unichar_isxdigit (ch);
 }
 
+
 static gboolean
 is_graph_keyval (guint keyval)
 {
@@ -719,6 +923,7 @@ is_graph_keyval (guint keyval)
   return g_unichar_isgraph (ch);
 }
 
+
 static void
 ibus_engine_simple_update_lookup_and_aux_table (IBusEngineSimple *simple)
 {
@@ -744,6 +949,7 @@ ibus_engine_simple_update_lookup_and_aux_table (IBusEngineSimple *simple)
                                      priv->lookup_table_visible);
 }
 
+
 static gboolean
 ibus_engine_simple_if_in_range_of_lookup_table (IBusEngineSimple *simple,
                                                 guint             keyval)
@@ -770,6 +976,7 @@ ibus_engine_simple_if_in_range_of_lookup_table (IBusEngineSimple *simple,
     return TRUE;
 }
 
+
 static void
 ibus_engine_simple_set_number_on_lookup_table (IBusEngineSimple *simple,
                                                guint             keyval,
@@ -816,6 +1023,7 @@ ibus_engine_simple_check_all_compose_table (IBusEngineSimple *simple,
     GString *output = g_string_new ("");
     gboolean success = FALSE;
     gboolean is_32bit = FALSE;
+    gboolean can_load_en_us = FALSE;
     gunichar output_char = '\0';
 
     /* GtkIMContextSimple output the first compose char in case of
@@ -827,27 +1035,40 @@ ibus_engine_simple_check_all_compose_table (IBusEngineSimple *simple,
     G_LOCK (global_tables);
     tmp_list = global_tables;
     while (tmp_list) {
+        IBusComposeTableEx *compose_table = tmp_list->data;
+        if (compose_table->can_load_en_us)
+            can_load_en_us = TRUE;
+        /* If global_tables includes en_compose_table only, i.e. no user
+         * or locale compose tables, en_compose_table is used.
+         * If not, en_compose_table is used in case one of the other compose
+         * tables has can_load_en_us = %TRUE, i.e. the table file has
+         * the line of 'include "%L"'.
+         * en_compose_table is always appended to the last of global_tables.
+         */
+        if ((compose_table == en_compose_table) && global_tables->next != NULL
+            && !can_load_en_us) {
+            tmp_list = tmp_list->next;
+            continue;
+        }
         is_32bit = FALSE;
-        if (ibus_compose_table_check (
-            (IBusComposeTableEx *)tmp_list->data,
-            priv->compose_buffer,
-            n_compose,
-            &compose_finish,
-            &compose_match,
-            output,
-            is_32bit)) {
+        if (ibus_compose_table_check (compose_table,
+                                      priv->compose_buffer,
+                                      n_compose,
+                                      &compose_finish,
+                                      &compose_match,
+                                      output,
+                                      is_32bit)) {
             success = TRUE;
             break;
         }
         is_32bit = TRUE;
-        if (ibus_compose_table_check (
-            (IBusComposeTableEx *)tmp_list->data,
-            priv->compose_buffer,
-            n_compose,
-            &compose_finish,
-            &compose_match,
-            output,
-            is_32bit)) {
+        if (ibus_compose_table_check (compose_table,
+                                      priv->compose_buffer,
+                                      n_compose,
+                                      &compose_finish,
+                                      &compose_match,
+                                      output,
+                                      is_32bit)) {
             success = TRUE;
             break;
         }
@@ -862,8 +1083,9 @@ ibus_engine_simple_check_all_compose_table (IBusEngineSimple *simple,
                 if (is_32bit) {
                     ibus_engine_simple_commit_str (simple, output->str);
                 } else {
-                    ibus_engine_simple_commit_char (simple,
-                                                    g_utf8_get_char (output->str));
+                    ibus_engine_simple_commit_char (
+                            simple,
+                            g_utf8_get_char (output->str));
                 }
             }
             ibus_engine_simple_update_preedit_text (simple);
@@ -927,8 +1149,10 @@ ibus_engine_simple_process_key_event (IBusEngine *engine,
     guint printable_keyval;
     int i;
 
-    while (n_compose <= COMPOSE_BUFFER_SIZE && priv->compose_buffer[n_compose] != 0)
+    while (n_compose <= COMPOSE_BUFFER_SIZE &&
+           priv->compose_buffer[n_compose] != 0) {
         n_compose++;
+    }
     if (n_compose > COMPOSE_BUFFER_SIZE) {
         g_warning ("copmose table buffer is full.");
         n_compose = COMPOSE_BUFFER_SIZE;
@@ -948,7 +1172,10 @@ ibus_engine_simple_process_key_event (IBusEngine *engine,
                 priv->modifiers_dropped = TRUE;
             } else {
                 /* invalid hex sequence */
-                /* FIXME beep_window (event->window); */
+                ibus_engine_simple_send_message_with_code (
+                        simple,
+                        IBUS_ENGINE_MSG_CODE_INVALID_COMPOSE_SEQUENCE,
+                        NULL);
                 g_string_set_size (priv->tentative_match, 0);
                 g_clear_pointer (&priv->tentative_emoji, g_free);
                 priv->in_hex_sequence = FALSE;
@@ -971,7 +1198,10 @@ ibus_engine_simple_process_key_event (IBusEngine *engine,
                 priv->modifiers_dropped = TRUE;
             } else {
                 /* invalid hex sequence */
-                /* FIXME beep_window (event->window); */
+                ibus_engine_simple_send_message_with_code (
+                        simple,
+                        IBUS_ENGINE_MSG_CODE_INVALID_COMPOSE_SEQUENCE,
+                        NULL);
                 g_string_set_size (priv->tentative_match, 0);
                 g_clear_pointer (&priv->tentative_emoji, g_free);
                 priv->in_hex_sequence = FALSE;
@@ -1020,7 +1250,10 @@ ibus_engine_simple_process_key_event (IBusEngine *engine,
         }
         else if (is_hex_end) {
             /* invalid hex sequence */
-            /* FIXME beep_window (event->window); */
+            ibus_engine_simple_send_message_with_code (
+                    simple,
+                    IBUS_ENGINE_MSG_CODE_INVALID_COMPOSE_SEQUENCE,
+                    NULL);
             g_string_set_size (priv->tentative_match, 0);
             g_clear_pointer (&priv->tentative_emoji, g_free);
             priv->in_hex_sequence = FALSE;
@@ -1119,7 +1352,10 @@ ibus_engine_simple_process_key_event (IBusEngine *engine,
         else {
             /* invalid hex sequence */
             if (n_compose > 0) {
-                /* FIXME beep_window (event->window); */
+                ibus_engine_simple_send_message_with_code (
+                        simple,
+                        IBUS_ENGINE_MSG_CODE_INVALID_COMPOSE_SEQUENCE,
+                        NULL);
                 g_string_set_size (priv->tentative_match, 0);
                 priv->in_hex_sequence = FALSE;
                 priv->compose_buffer[0] = 0;
@@ -1184,7 +1420,10 @@ ibus_engine_simple_process_key_event (IBusEngine *engine,
             return TRUE;
         } else if (!is_hex_end) {
             /* non-hex character in hex sequence */
-            /* FIXME beep_window (event->window); */
+            ibus_engine_simple_send_message_with_code (
+                    simple,
+                    IBUS_ENGINE_MSG_CODE_INVALID_COMPOSE_SEQUENCE,
+                    NULL);
             return TRUE;
         }
     } else if (priv->in_emoji_sequence) {
@@ -1243,15 +1482,20 @@ ibus_engine_simple_process_key_event (IBusEngine *engine,
                     return TRUE;
                 } else {
                     /* invalid hex sequence */
-                    /* FIXME beep_window (event->window); */
+                    ibus_engine_simple_send_message_with_code (
+                            simple,
+                            IBUS_ENGINE_MSG_CODE_INVALID_COMPOSE_SEQUENCE,
+                            NULL);
                     g_string_set_size (priv->tentative_match, 0);
                     priv->in_hex_sequence = FALSE;
                     priv->compose_buffer[0] = 0;
                 }
+            } else if (!check_hex (simple, n_compose)) {
+                ibus_engine_simple_send_message_with_code (
+                        simple,
+                        IBUS_ENGINE_MSG_CODE_INVALID_COMPOSE_SEQUENCE,
+                        NULL);
             }
-            else if (!check_hex (simple, n_compose))
-                /* FIXME beep_window (event->window); */
-                ;
             ibus_engine_simple_update_preedit_text (simple);
 
             return TRUE;
@@ -1343,7 +1587,10 @@ ibus_engine_simple_process_key_event (IBusEngine *engine,
 
             n_compose = n_compose_prev;
             g_assert (n_compose < (COMPOSE_BUFFER_SIZE + 1));
-            /* FIXME beep_window (event->window); */
+            ibus_engine_simple_send_message_with_code (
+                    simple,
+                    IBUS_ENGINE_MSG_CODE_INVALID_COMPOSE_SEQUENCE,
+                    NULL);
             backup_char = priv->compose_buffer[n_compose];
             priv->compose_buffer[n_compose] = 0;
             if (ibus_engine_simple_check_all_compose_table (simple, n_compose))
@@ -1356,6 +1603,7 @@ ibus_engine_simple_process_key_event (IBusEngine *engine,
     return no_sequence_matches (simple, n_compose, keyval, keycode, modifiers);
 }
 
+
 static void
 ibus_engine_simple_page_down (IBusEngine *engine)
 {
@@ -1367,6 +1615,7 @@ ibus_engine_simple_page_down (IBusEngine *engine)
     ibus_engine_simple_update_lookup_and_aux_table (simple);
 }
 
+
 static void
 ibus_engine_simple_page_up (IBusEngine *engine)
 {
@@ -1378,6 +1627,7 @@ ibus_engine_simple_page_up (IBusEngine *engine)
     ibus_engine_simple_update_lookup_and_aux_table (simple);
 }
 
+
 static void
 ibus_engine_simple_candidate_clicked (IBusEngine *engine,
                                       guint       index,
@@ -1401,6 +1651,7 @@ ibus_engine_simple_candidate_clicked (IBusEngine *engine,
     ibus_engine_simple_set_number_on_lookup_table (simple, keyval, n_compose);
 }
 
+
 void
 ibus_engine_simple_add_table (IBusEngineSimple *simple,
                               const guint16    *data,
@@ -1415,6 +1666,7 @@ ibus_engine_simple_add_table (IBusEngineSimple *simple,
                                                        n_seqs);
 }
 
+
 gboolean
 ibus_engine_simple_add_table_by_locale (IBusEngineSimple *simple,
                                         const gchar      *locale)
@@ -1533,13 +1785,24 @@ ibus_engine_simple_add_table_by_locale (IBusEngineSimple *simple,
     return retval;
 }
 
+
 gboolean
 ibus_engine_simple_add_compose_file (IBusEngineSimple *simple,
                                      const gchar      *compose_file)
 {
-    g_return_val_if_fail (IBUS_IS_ENGINE_SIMPLE (simple), TRUE);
+    GError *error = NULL;
+
+    g_return_val_if_fail (IBUS_IS_ENGINE_SIMPLE (simple), FALSE);
 
     global_tables = ibus_compose_table_list_add_file (global_tables,
-                                                      compose_file);
+                                                      compose_file,
+                                                      &error);
+    if (error) {
+        g_warning ("\n%s\n", error->message);
+        ibus_engine_simple_send_message_with_code (simple,
+                                                   error->code,
+                                                   error);
+        g_error_free (error);
+    }
     return TRUE;
 }
