@@ -7,17 +7,17 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
+ * USA
  */
 #include "ibuslookuptable.h"
 
@@ -105,7 +105,15 @@ ibus_lookup_table_serialize (IBusLookupTable *table,
         IBusText *text = ibus_lookup_table_get_candidate (table, i);
         if (text == NULL)
             break;
-        g_variant_builder_add (&array, "v", ibus_serializable_serialize ((IBusSerializable *)text));
+        /* Replaced g_variant_builder_add() with g_variant_builder_open() &
+         * g_variant_builder_close() to avoid creating temporary objects during
+         * serialization.
+         */
+        g_variant_builder_open (&array, G_VARIANT_TYPE_VARIANT);
+        g_variant_builder_add_value (
+                &array,
+                ibus_serializable_serialize ((IBusSerializable *)text));
+        g_variant_builder_close (&array);
     }
     g_variant_builder_add (builder, "av", &array);
 
@@ -116,7 +124,11 @@ ibus_lookup_table_serialize (IBusLookupTable *table,
         if (text == NULL)
             break;
 
-        g_variant_builder_add (&array, "v", ibus_serializable_serialize ((IBusSerializable *)text));
+        g_variant_builder_open (&array, G_VARIANT_TYPE_VARIANT);
+        g_variant_builder_add_value (
+                &array,
+                ibus_serializable_serialize ((IBusSerializable *)text));
+        g_variant_builder_close (&array);
     }
     g_variant_builder_add (builder, "av", &array);
 
